@@ -3,6 +3,8 @@
     ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 """
 
+from __future__ import absolute_import
+
 __all__ = ['Error', 'NonexistentTarget']
 
 
@@ -11,5 +13,5 @@ class Error(LookupError):
 
 
 class NonexistentTarget(Error):
-    """raised when trying to get a non existent target"""
+    """Raised when trying to get a non existent target"""
     pass
