@@ -5,6 +5,8 @@
     Implements JSON Schema draft04.
 """
 
+from __future__ import absolute_import
+
 __all__ = ['compile', 'Draft04Validator']
 
 import logging
@@ -396,7 +398,7 @@ class Draft04Validator(Validator):
         ================= ============
         Expected draft04  Alias of
         ----------------- ------------
-        date-time         rfc3339:datetime
+        date-time         rfc3339.datetime
         email             email
         hostname          hostname
         ipv4              ipv4
@@ -407,7 +409,7 @@ class Draft04Validator(Validator):
         """
         if 'format' in self.attrs:
             substituted = {
-                'date-time': 'rfc3339:datetime',
+                'date-time': 'rfc3339.datetime',
                 'email': 'email',
                 'hostname': 'hostname',
                 'ipv4': 'ipv4',
